-------------------------------------------------------------------------------
-- Title      : 10GMAC FIFO
-- Project    : NFC_MEAS
-------------------------------------------------------------------------------
-- File       : interface_fifo_10gmac.vhd
-- Author     : Dominik Seidl  <dominik.seidl@ic-design.de>
-- Company    : IC-Design Gottinger GmbH
-- Created    : 2017-10-05
-- Last update: 2017-10-05
-- Platform   : RHEL 5
-- Standard   : VHDL'93
-------------------------------------------------------------------------------
-- Description: Interface Fifo for Microsemi 10GMAC
-------------------------------------------------------------------------------
-- Copyright (c) 2010 IC-Design Gottinger GmbH, Ilzleite 34, 94034 Passau
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author                  Description
-- 2017-10-05  1.0      Dominik Seidl	        Created
-------------------------------------------------------------------------------

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;

ENTITY handshake_sync IS
PORT(
     clk_a_i    : IN  STD_LOGIC;
     clk_b_i    : IN  STD_LOGIC;
     reset_i   	: IN  STD_LOGIC;
     signal_a_i : IN  STD_LOGIC;
	 ack_a_o	: OUT STD_LOGIC;
     signal_b_o : OUT STD_LOGIC
     );
END handshake_sync;

ARCHITECTURE rtl OF handshake_sync IS

  SIGNAL a_buffer1    	: STD_LOGIC := '0';
  SIGNAL a_buffer2    	: STD_LOGIC := '0';
  SIGNAL a_buffer3    	: STD_LOGIC := '0';
  SIGNAL b_buffer1   	: STD_LOGIC := '0';
  SIGNAL b_buffer2   	: STD_LOGIC := '0';
  
BEGIN
  
  PROCESS (clk_a_i, reset_i)
  BEGIN
    IF reset_i = '0' THEN
        a_buffer1    <= '0';
        a_buffer2    <= '0';
		a_buffer3    <= '0';
    ELSIF clk_a_i'EVENT AND clk_a_i = '1' THEN
		a_buffer1    <= signal_a_i;
		a_buffer2    <= b_buffer2;
		a_buffer3    <= a_buffer2;
    END IF;
  END PROCESS;
  
  ack_a_o	<= a_buffer3;

  PROCESS (clk_b_i, reset_i)
  BEGIN
    IF reset_i = '0' THEN
        b_buffer1    <= '0';
        b_buffer2    <= '0';
    ELSIF clk_b_i'EVENT AND clk_b_i = '1' THEN
        b_buffer1    <= a_buffer1;
		b_buffer2    <= b_buffer1;
    END IF;
  END PROCESS;
  
  signal_b_o   <= b_buffer2;
  
END rtl;

LIBRARY IEEE;
USE IEEE.STD_LOGIC_1164.ALL;
USE IEEE.STD_LOGIC_ARITH.ALL;

ENTITY interface_fifo_10gmac IS
PORT(-- APB Bus
     PCLK               : IN  STD_LOGIC;
     PRESETn            : IN  STD_LOGIC;
     PADDR              : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
     PSEL               : IN  STD_LOGIC;
     PENABLE            : IN  STD_LOGIC;
     PWRITE             : IN  STD_LOGIC;
     PWDATA             : IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
     PREADY             : OUT STD_LOGIC;
     PRDATA             : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
     PSLVERR            : OUT STD_LOGIC;
	 
	 -- Interface Clock
	 SYSCLK             : IN  STD_LOGIC;

     -- Transmitt interface signals
	 tx_en              : OUT STD_LOGIC;
     tx_sop             : OUT STD_LOGIC;                
     tx_eop             : OUT STD_LOGIC;                
     tx_data            : OUT STD_LOGIC_VECTOR(63 DOWNTO 0);
     tx_bc              : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
     tx_stats           : IN  STD_LOGIC_VECTOR(39 DOWNTO 0);	 
	 
     -- Receive interface signals
	 rx_en              : IN  STD_LOGIC;
     rx_sop             : IN  STD_LOGIC;                
     rx_eop             : IN  STD_LOGIC;                
     rx_data            : IN  STD_LOGIC_VECTOR(63 DOWNTO 0);
     rx_bc              : IN  STD_LOGIC_VECTOR(7 DOWNTO 0);
     rx_err             : IN  STD_LOGIC;
     rx_stats           : IN  STD_LOGIC_VECTOR(39 DOWNTO 0)	 
     );
END interface_fifo_10gmac;

ARCHITECTURE rtl OF interface_fifo_10gmac IS

  COMPONENT handshake_sync
  PORT(
     clk_a_i    : IN  STD_LOGIC;
     clk_b_i    : IN  STD_LOGIC;
     reset_i    : IN  STD_LOGIC;
     signal_a_i : IN  STD_LOGIC;
     ack_a_o    : OUT STD_LOGIC;
     signal_b_o : OUT STD_LOGIC
     );
  END COMPONENT;
  
  COMPONENT gmac_fifo_apb
  PORT(
     DATA  		: IN  STD_LOGIC_VECTOR(74 DOWNTO 0);
     RCLOCK    	: IN  STD_LOGIC;
     RE    		: IN  STD_LOGIC;
     RESET 		: IN  STD_LOGIC;
     WCLOCK    	: IN  STD_LOGIC;
     WE 		: IN  STD_LOGIC;
	 DVLD       : OUT STD_LOGIC;
	 EMPTY 		: OUT STD_LOGIC;
	 FULL 		: OUT STD_LOGIC;
	 Q 			: OUT STD_LOGIC_VECTOR(74 DOWNTO 0)
     );
  END COMPONENT;
  
  COMPONENT stats_fifo
  PORT(
     DATA  		: IN  STD_LOGIC_VECTOR(31 DOWNTO 0);
     RCLOCK    	: IN  STD_LOGIC;
     RE    		: IN  STD_LOGIC;
     RESET 		: IN  STD_LOGIC;
     WCLOCK    	: IN  STD_LOGIC;
     WE 		: IN  STD_LOGIC;
	 EMPTY 		: OUT STD_LOGIC;
	 FULL 		: OUT STD_LOGIC;
	 Q 			: OUT STD_LOGIC_VECTOR(31 DOWNTO 0)
     );
  END COMPONENT;
  
  TYPE write_fifo_fsm_t IS (idle, read_from_fifo);
  SIGNAL write_fifo_state       : write_fifo_fsm_t; -- fsm current state 
  
  TYPE read_fifo_fsm_t IS (idle, write_to_fifo);
  SIGNAL read_fifo_state       : read_fifo_fsm_t; -- fsm current state 

  SIGNAL transmitt_reg_msb      : STD_LOGIC_VECTOR(31 DOWNTO 0);
  SIGNAL transmitt_reg_lsb      : STD_LOGIC_VECTOR(31 DOWNTO 0);
  SIGNAL transmitt_reg_ctrl     : STD_LOGIC_VECTOR(31 DOWNTO 0);
  
  SIGNAL transmitt_reg_ctrl_31_ack	: STD_LOGIC;
  SIGNAL transmitt_reg_ctrl_31_sync	: STD_LOGIC;
  
  SIGNAL receive_reg_msb        : STD_LOGIC_VECTOR(31 DOWNTO 0);
  SIGNAL receive_reg_lsb        : STD_LOGIC_VECTOR(31 DOWNTO 0);
  SIGNAL receive_reg_ctrl       : STD_LOGIC_VECTOR(31 DOWNTO 0);
  
  SIGNAL write_fifo_re    		: STD_LOGIC;
  SIGNAL write_fifo_we    		: STD_LOGIC;
  SIGNAL write_fifo_dvld 		: STD_LOGIC;
  SIGNAL write_fifo_empty 		: STD_LOGIC;
  SIGNAL write_fifo_full   		: STD_LOGIC;
  SIGNAL write_fifo_wdata  		: STD_LOGIC_VECTOR(74 DOWNTO 0);
  SIGNAL write_fifo_rdata  		: STD_LOGIC_VECTOR(74 DOWNTO 0);
  
  SIGNAL read_fifo_re    		: STD_LOGIC;
  SIGNAL read_fifo_we    		: STD_LOGIC;
  SIGNAL read_fifo_dvld 		: STD_LOGIC;
  SIGNAL read_fifo_empty 		: STD_LOGIC;
  SIGNAL read_fifo_full   		: STD_LOGIC;
  SIGNAL read_fifo_wdata  		: STD_LOGIC_VECTOR(74 DOWNTO 0);
  SIGNAL read_fifo_rdata  		: STD_LOGIC_VECTOR(74 DOWNTO 0);
  
  SIGNAL tx_stats_fifo_re    	: STD_LOGIC;
  SIGNAL tx_stats_fifo_we    	: STD_LOGIC;
  SIGNAL tx_stats_fifo_dvld 	: STD_LOGIC;
  SIGNAL tx_stats_fifo_empty 	: STD_LOGIC;
  SIGNAL tx_stats_fifo_full   	: STD_LOGIC;
  SIGNAL tx_stats_fifo_wdata  	: STD_LOGIC_VECTOR(31 DOWNTO 0);
  SIGNAL tx_stats_fifo_rdata  	: STD_LOGIC_VECTOR(31 DOWNTO 0);
  
  SIGNAL rx_stats_fifo_re    	: STD_LOGIC;
  SIGNAL rx_stats_fifo_we    	: STD_LOGIC;
  SIGNAL rx_stats_fifo_dvld 	: STD_LOGIC;
  SIGNAL rx_stats_fifo_empty 	: STD_LOGIC;
  SIGNAL rx_stats_fifo_full   	: STD_LOGIC;
  SIGNAL rx_stats_fifo_wdata  	: STD_LOGIC_VECTOR(31 DOWNTO 0);
  SIGNAL rx_stats_fifo_rdata  	: STD_LOGIC_VECTOR(31 DOWNTO 0);
  
  SIGNAL mac_addr               : STD_LOGIC_VECTOR(47 DOWNTO 0);
  SIGNAL mac_addr_high_reg      : STD_LOGIC_VECTOR(31 DOWNTO 0);
  SIGNAL mac_addr_low_reg       : STD_LOGIC_VECTOR(31 DOWNTO 0);  
  
  SIGNAL next_pready            : STD_LOGIC;
  SIGNAL pready_int             : STD_LOGIC;
  
  CONSTANT mac_addr_broadcast   : STD_LOGIC_VECTOR(47 DOWNTO 0) := X"FFFFFFFFFFFF";
  
BEGIN

  write_fifo: gmac_fifo_apb
  PORT MAP(
     DATA  		=> write_fifo_wdata,
     RCLOCK    	=> SYSCLK, 
     RE    		=> write_fifo_re,
     RESET 		=> PRESETn,
     WCLOCK    	=> PCLK,
     WE 		=> write_fifo_we,
	 DVLD		=> write_fifo_dvld,
	 EMPTY 		=> write_fifo_empty,
	 FULL 		=> write_fifo_full,
	 Q 			=> write_fifo_rdata
     );
	 
  read_fifo: gmac_fifo_apb
  PORT MAP(
     DATA  		=> read_fifo_wdata,
     RCLOCK    	=> PCLK, 
     RE    		=> read_fifo_re,
     RESET 		=> PRESETn,
     WCLOCK    	=> SYSCLK,
     WE 		=> read_fifo_we,
	 DVLD		=> read_fifo_dvld,
	 EMPTY 		=> read_fifo_empty,
	 FULL 		=> read_fifo_full,
	 Q 			=> read_fifo_rdata
     );	 
	 
	 
  tx_stats_fifo: stats_fifo
  PORT MAP(
     DATA  		=> tx_stats_fifo_wdata,
     RCLOCK    	=> PCLK, 
     RE    		=> tx_stats_fifo_re,
     RESET 		=> PRESETn,
     WCLOCK    	=> SYSCLK,
     WE 		=> tx_stats_fifo_we,
	 EMPTY 		=> tx_stats_fifo_empty,
	 FULL 		=> tx_stats_fifo_full,
	 Q 			=> tx_stats_fifo_rdata
     );	 

  rx_stats_fifo: stats_fifo
  PORT MAP(
     DATA  		=> rx_stats_fifo_wdata,
     RCLOCK    	=> PCLK, 
     RE    		=> rx_stats_fifo_re,
     RESET 		=> PRESETn,
     WCLOCK    	=> SYSCLK,
     WE 		=> rx_stats_fifo_we,
	 EMPTY 		=> rx_stats_fifo_empty,
	 FULL 		=> rx_stats_fifo_full,
	 Q 			=> rx_stats_fifo_rdata
     );		 
	 
	 
  config_reg_sync_0: handshake_sync
  PORT MAP(
     clk_a_i    => PCLK,
     clk_b_i    => SYSCLK,
     reset_i    => PRESETn,
     signal_a_i => transmitt_reg_ctrl(31),
     ack_a_o    => transmitt_reg_ctrl_31_ack,
     signal_b_o => transmitt_reg_ctrl_31_sync
     );   	 

  Config_Regs: PROCESS(PCLK, PRESETn)
  BEGIN
    IF PRESETn = '0' THEN
      transmitt_reg_msb   	<= (OTHERS => '0'); -- reset REGISTER
	  transmitt_reg_lsb     <= (OTHERS => '0'); -- reset REGISTER
	  transmitt_reg_ctrl    <= (OTHERS => '0'); -- reset REGISTER
    ELSIF PCLK'EVENT AND PCLK = '1' THEN
      IF (PWRITE = '1' AND PENABLE = '1' AND PSEL = '1' AND PADDR(5 DOWNTO 0) = "000000") THEN
        transmitt_reg_msb 	<= PWDATA(31 DOWNTO 0);
      ELSIF (PWRITE = '1' AND PENABLE = '1' AND PSEL = '1' AND PADDR(5 DOWNTO 0) = "000100") THEN
        transmitt_reg_lsb	<= PWDATA(31 DOWNTO 0);
      ELSIF (PWRITE = '1' AND PENABLE = '1' AND PSEL = '1' AND PADDR(5 DOWNTO 0) = "001000") THEN
        transmitt_reg_ctrl  <= PWDATA(31 DOWNTO 0);
        -- write to fifo
		IF write_fifo_full = '0' THEN
		   write_fifo_we	<= '1';
		   write_fifo_wdata	<= PWDATA(10 DOWNTO 0) & transmitt_reg_msb & transmitt_reg_lsb;
		END IF;
		
	  ELSIF (PWRITE = '1' AND PENABLE = '1' AND PSEL = '1' AND PADDR(5 DOWNTO 0) = "101000") THEN
        mac_addr_high_reg   <= PWDATA(31 DOWNTO 0); 
	  ELSIF (PWRITE = '1' AND PENABLE = '1' AND PSEL = '1' AND PADDR(5 DOWNTO 0) = "101100") THEN
        mac_addr_low_reg    <= PWDATA(31 DOWNTO 0); 
		
      ELSIF transmitt_reg_ctrl_31_ack = '1' THEN
	    transmitt_reg_ctrl(31)	<= '0';
	  ELSE
	    write_fifo_we		<= '0';
      END IF;     
    END IF;
  END PROCESS Config_Regs;

  -- combinational process to evaluate the incoming paddr
  eval_paddr: PROCESS(PSEL, PWRITE, PADDR, read_fifo_empty, tx_stats_fifo_rdata, rx_stats_fifo_rdata,
                      receive_reg_msb, receive_reg_lsb, receive_reg_ctrl, read_fifo_rdata, write_fifo_empty)
  BEGIN
    -- default
    
    IF PSEL = '1' AND PWRITE = '1' THEN
      next_pready       <= '1';
      PRDATA            <= (OTHERS => '0');
    ELSIF PSEL = '1' AND PWRITE = '0' AND PADDR(5 DOWNTO 0) = "001100" THEN
      next_pready       <= '1';
      PRDATA            <= receive_reg_msb;
    ELSIF PSEL = '1' AND PWRITE = '0' AND PADDR(5 DOWNTO 0) = "010000" THEN
      next_pready       <= '1';
      PRDATA            <= receive_reg_lsb;
    ELSIF PSEL = '1' AND PWRITE = '0' AND PADDR(5 DOWNTO 0) = "010100" THEN
      next_pready       <= '1';
      PRDATA            <= X"00000" & "00" & read_fifo_rdata(73 DOWNTO 64);  
    ELSIF PSEL = '1' AND PWRITE = '0' AND PADDR(5 DOWNTO 0) = "011000" THEN
      next_pready       <= '1';
      PRDATA            <= X"0000000" & "00" & write_fifo_empty & read_fifo_empty; 	

    ELSIF PSEL = '1' AND PWRITE = '0' AND PADDR(5 DOWNTO 0) = "011100" THEN
      next_pready       <= '1';
      PRDATA            <= tx_stats_fifo_rdata; 
    ELSIF PSEL = '1' AND PWRITE = '0' AND PADDR(5 DOWNTO 0) = "100000" THEN
      next_pready       <= '1';
      PRDATA            <= rx_stats_fifo_rdata; 
    ELSIF PSEL = '1' AND PWRITE = '0' AND PADDR(5 DOWNTO 0) = "100100" THEN
      next_pready       <= '1';
      PRDATA            <= X"0000000" & "00" & tx_stats_fifo_empty & rx_stats_fifo_empty;	  
      
    -- read from an unused address  
    ELSIF PSEL = '1' AND PWRITE = '0' AND PADDR(5 DOWNTO 0) /= "010100" THEN
      next_pready       <= '1' ;
      PRDATA            <= (OTHERS => '0');
    ELSE
      next_pready       <= '0' ;
      PRDATA            <= (OTHERS => '0');
    END IF;
  END PROCESS eval_paddr;

  -- combinational process to evaluate the incoming paddr
  fifo_rd_eval: PROCESS(PSEL, PWRITE, PADDR, PENABLE)
  BEGIN
    -- default
    read_fifo_re        <= '0';
	tx_stats_fifo_re    <= '0';
	rx_stats_fifo_re    <= '0';
    IF PSEL = '1' AND PWRITE = '0' AND PADDR(5 DOWNTO 0) = "010100" AND PENABLE = '0' THEN
      read_fifo_re      <= '1';
    END IF;
	IF PSEL = '1' AND PWRITE = '0' AND PADDR(5 DOWNTO 0) = "011100" AND PENABLE = '0' THEN
      tx_stats_fifo_re  <= '1';
    END IF;
	IF PSEL = '1' AND PWRITE = '0' AND PADDR(5 DOWNTO 0) = "100000" AND PENABLE = '0' THEN
      rx_stats_fifo_re  <= '1';
    END IF;
  END PROCESS fifo_rd_eval;

  -- clock out data
  clock_out: PROCESS(PCLK, PRESETn)
  BEGIN
    --async. reset
    IF PRESETn = '0' THEN
      pready_int        	<= '0' ;
      receive_reg_msb       <= (OTHERS => '0'); -- reset REGISTER
	  receive_reg_lsb       <= (OTHERS => '0'); -- reset REGISTER	
      receive_reg_ctrl      <= (OTHERS => '0'); -- reset REGISTER	  
    ELSIF PCLK'EVENT AND PCLK = '1' THEN
      --data are fetched, synchronous reset
      IF PENABLE = '1' AND pready_int = '1' THEN
        pready_int      <= '0';
		IF PADDR(5 DOWNTO 0) = "010100" THEN
		   receive_reg_msb	<= read_fifo_rdata(63 DOWNTO 32);
		   receive_reg_lsb	<= read_fifo_rdata(31 DOWNTO 0);
		   receive_reg_ctrl <= X"00000" & '0' & read_fifo_rdata(74 DOWNTO 64);
		END IF;
      ELSE
        pready_int      <= next_pready;
      END IF ;
    END IF ;
  END PROCESS clock_out ;

  PREADY        <= pready_int;
  PSLVERR       <= '0';                                                           

  --------------------------------------------------
  -- Write FIFO State Machine
  --------------------------------------------------
  write_fifo_fsm: PROCESS(SYSCLK, PRESETn)
  BEGIN
    IF PRESETn = '0' THEN
      write_fifo_state          <= idle;
      write_fifo_re             <= '0';
    ELSIF SYSCLK'EVENT AND SYSCLK = '1' THEN
      CASE write_fifo_state IS
        WHEN idle          	=> IF transmitt_reg_ctrl_31_sync = '1' THEN   
                                 write_fifo_state 	<= read_from_fifo;
                                 write_fifo_re    	<= '1';                              
                               ELSE
                                 write_fifo_state 	<= idle;
                               END IF;
        -----------------------------------------------
        -- Read from Write FIFO
        -----------------------------------------------
        WHEN read_from_fifo	=> IF write_fifo_empty = '1' THEN            
                                 write_fifo_state 	<= idle;
                                 write_fifo_re  	<= '0';         
                               ELSE
								 write_fifo_state  	<= read_from_fifo;
                               END IF;                          
        WHEN OTHERS         => NULL;
      END CASE;
    END IF;
  END PROCESS write_fifo_fsm;
  
  --------------------------------------------------
  -- Read FIFO State Machine
  --------------------------------------------------
  mac_addr <= mac_addr_high_reg & mac_addr_low_reg(15 DOWNTO 0);
  
  read_fifo_fsm: PROCESS(SYSCLK, PRESETn)
  BEGIN
    IF PRESETn = '0' THEN
      read_fifo_state          <= idle;
      read_fifo_we             <= '0';
	  read_fifo_wdata		   <= (OTHERS => '0');
    ELSIF SYSCLK'EVENT AND SYSCLK = '1' THEN
      CASE read_fifo_state IS
        WHEN idle          	=> IF rx_en = '1' AND rx_sop = '1' THEN 
                                 -- if MAC filtering is enabled
                                 IF (mac_addr_low_reg(31) = '1') AND 
								    ((rx_data(63 DOWNTO 16) = mac_addr) OR 
									 (rx_data(63 DOWNTO 16) = mac_addr_broadcast)) THEN
								   read_fifo_state 	<= write_to_fifo;
                                   read_fifo_we    	<= '1'; 	 
                                   read_fifo_wdata	<= rx_err & rx_sop & rx_eop & rx_bc & rx_data;
                                 ELSIF 	(mac_addr_low_reg(31) = '0') THEN
								   read_fifo_state 	<= write_to_fifo;
                                   read_fifo_we    	<= '1'; 	 
                                   read_fifo_wdata	<= rx_err & rx_sop & rx_eop & rx_bc & rx_data;
								 END IF;                             
                               ELSE
                                 read_fifo_state 	<= idle;
								 read_fifo_we       <= '0';
	                             read_fifo_wdata    <= (OTHERS => '0');
                               END IF;
        -----------------------------------------------
        -- Read from Write FIFO
        -----------------------------------------------
        WHEN write_to_fifo	=> IF rx_en = '1' THEN 
                                 read_fifo_we    	<= '1'; 	 
                                 read_fifo_wdata	<= rx_err & rx_sop & rx_eop & rx_bc & rx_data;
                                 IF rx_eop = '1' THEN								 
                                   read_fifo_state 	<= idle;
                                 END IF;								   
                               ELSE
								 read_fifo_we       <= '0';
	                             read_fifo_wdata    <= (OTHERS => '0');
                               END IF;                          
        WHEN OTHERS         => NULL;
      END CASE;
    END IF;
  END PROCESS read_fifo_fsm;  
  
  -- Write to Read Fifo                                                      
  --read_fifo_wdata	<= rx_err & rx_sop & rx_eop & rx_bc & rx_data; 
  --read_fifo_we		<= rx_en WHEN (read_fifo_full = '0') ELSE '0';
  
  -- Read from Write Fifo
  --write_fifo_re		<= '1' WHEN write_fifo_empty = '0' ELSE '0';
  tx_en				<= write_fifo_dvld;
  tx_sop			<= write_fifo_rdata(73) WHEN write_fifo_dvld = '1' ELSE '0';
  tx_eop			<= write_fifo_rdata(72) WHEN write_fifo_dvld = '1' ELSE '0';
  tx_bc 			<= write_fifo_rdata(71 DOWNTO 64) WHEN write_fifo_dvld = '1' ELSE (OTHERS => '0');
  tx_data			<= write_fifo_rdata(63 DOWNTO 0) WHEN write_fifo_dvld = '1' ELSE (OTHERS => '0');
  
  -- Write to Tx Stats Fifo                                                      
  tx_stats_fifo_wdata	<= tx_stats(31 DOWNTO 0); 
  tx_stats_fifo_we	    <= tx_stats(31);
  -- Write to Rx Stats Fifo                                                      
  rx_stats_fifo_wdata	<= rx_stats(31 DOWNTO 0); 
  rx_stats_fifo_we	    <= rx_stats(31);
  
END rtl;